@@ -12,3 +12,10 @@ foo1 x = proc (y, z) -> do
       (bindA -< y)
     |)
     z
+
+foo2 = proc () -> do
+  ( proc () ->
+      returnA -< ()
+    )
+    -<
+      ()
