@@ -4,8 +4,7 @@ foo f = proc a -> (f -< a)
 
 bar f g = proc a ->
   ( ( (f)
-        ( g
-        )
+        (g)
     )
       -<
         ( ( ( ( ( ( g
