@@ -22,5 +22,17 @@ barbaz x y z w = [
   ]
 
 a = do
+  d <-
+    [ x + 1
+      | x <- b
+        ]
+
   [ c
       | c <- d ]
+
+trans =
+  [ x
+  | x <- xs
+  , then reverse
+  , then reverse
+  ]
