@@ -1,6 +1,7 @@
 import qualified MegaModule as M
   ( -- (1)
-    (<<<), -- (2)
-    (>>>),
+    -- (2)
     Either, -- (3)
+    (<<<),
+    (>>>),
   )
