@@ -1,5 +1,2 @@
 -- | This demonstrates a BUG.
-module Foo
-  (
-  )
-where
+module Foo () where
