@@ -45,6 +45,17 @@ import Text.Megaparsec.Char.Lexer qualified as L
 
 type Parser = Parsec Void Text
 
+-- TODO Support fixity namespacing?
+-- https://github.com/ghc-proposals/ghc-proposals/blob/master/proposals/0065-type-infix.rst
+--
+-- Note that currently, our fixity machinery does *not* do any namespacing:
+--
+--  - https://github.com/tweag/ormolu/pull/994#pullrequestreview-1396958951
+--    brought this up in the past
+--
+--  - https://github.com/tweag/ormolu/pull/1029#issue-1718217029
+--    has a concrete example (morley-prelude) where namespacing would matter
+
 -- | Parse textual representation of 'FixityOverrides'.
 parseDotOrmolu ::
   -- | Location of the file we are parsing (only for parse errors)
@@ -79,14 +90,10 @@ pDotOrmolu =
     (FixityOverrides . Map.fromList . mconcat)
     (ModuleReexports . Map.map NE.sort . Map.fromListWith (<>))
     . partitionEithers
-    <$> many configLine
+    <$> (configLine `sepEndBy` (void eol *> hidden space))
     <* eof
   where
-    configLine = do
-      x <- eitherP pFixity pModuleReexport
-      void eol
-      hidden space
-      return x
+    configLine = eitherP pFixity pModuleReexport
 
 -- | Parse a single fixity declaration, such as
 --
@@ -96,7 +103,9 @@ pFixity = do
   fiDirection <- pFixityDirection
   hidden hspace1
   offsetAtPrecedence <- getOffset
-  fiPrecedence <- L.decimal
+  fiPrecedence <-
+    try L.float
+      <|> (fromIntegral <$> (L.decimal :: Parser Integer))
   when (fiPrecedence > 9) $
     region
       (setErrorOffset offsetAtPrecedence)
