@@ -28,7 +28,6 @@ import GHC.Data.EnumSet qualified as EnumSet
 import GHC.Data.FastString qualified as GHC
 import GHC.Data.Maybe (orElse)
 import GHC.Data.StringBuffer (StringBuffer)
-import GHC.Driver.CmdLine qualified as GHC
 import GHC.Driver.Config.Parser (initParserOpts)
 import GHC.Driver.Errors.Types qualified as GHC
 import GHC.Driver.Session as GHC
@@ -44,7 +43,6 @@ import GHC.Types.SourceError qualified as GHC
 import GHC.Types.SrcLoc
 import GHC.Utils.Error
 import GHC.Utils.Exception (ExceptionMonad)
-import GHC.Utils.Outputable (defaultSDocContext)
 import GHC.Utils.Panic qualified as GHC
 import Ormolu.Config
 import Ormolu.Exception
@@ -70,7 +68,7 @@ parseModule ::
   -- | Input for parser
   Text ->
   m
-    ( [GHC.Warn],
+    ( GHC.DriverMessages,
       Either (SrcSpan, String) [SourceSnippet]
     )
 parseModule config@Config {..} packageFixityMap path rawInput = liftIO $ do
@@ -134,7 +132,7 @@ parseModuleSnippet Config {..} modFixityMap dynFlags path rawInput = liftIO $ do
                   Nothing -> ""
                 msg =
                   showOutputable
-                    . formatBulleted defaultSDocContext
+                    . formatBulleted
                     . diagnosticMessage GHC.NoDiagnosticOpts
                     $ err
          in case L.sortOn (rateSeverity . errMsgSeverity) errs of
@@ -177,7 +175,11 @@ parseModuleSnippet Config {..} modFixityMap dynFlags path rawInput = liftIO $ do
 normalizeModule :: HsModule GhcPs -> HsModule GhcPs
 normalizeModule hsmod =
   everywhere
-    (mkT dropBlankTypeHaddocks `extT` dropBlankDataDeclHaddocks `extT` patchContext)
+    ( mkT dropBlankTypeHaddocks
+        `extT` dropBlankDataDeclHaddocks
+        `extT` patchContext
+        `extT` patchExprContext
+    )
     hsmod
       { hsmodImports =
           normalizeImports (hsmodImports hsmod),
@@ -211,11 +213,18 @@ normalizeModule hsmod =
         | isBlankDocString s -> ConDeclH98 {con_doc = Nothing, ..}
       a -> a
 
+    -- For constraint contexts (both in types and in expressions), normalize
+    -- parenthesis as decided in https://github.com/tweag/ormolu/issues/264.
     patchContext :: LHsContext GhcPs -> LHsContext GhcPs
     patchContext = fmap $ \case
       [x@(L _ (HsParTy _ _))] -> [x]
-      [x@(L lx _)] -> [L lx (HsParTy EpAnnNotUsed x)]
+      [x@(L lx _)] -> [L lx (HsParTy noAnn x)]
       xs -> xs
+    patchExprContext :: LHsExpr GhcPs -> LHsExpr GhcPs
+    patchExprContext = fmap $ \case
+      x@(HsQual _ (L _ [L _ HsPar {}]) _) -> x
+      HsQual l0 (L l1 [x@(L lx _)]) e -> HsQual l0 (L l1 [L lx (HsPar noAnn x)]) e
+      x -> x
 
 -- | Enable all language extensions that we think should be enabled by
 -- default for ease of use.
@@ -254,7 +263,9 @@ manualExts =
     LinearTypes, -- steals the (%) type operator in some cases
     OverloadedRecordDot, -- f.g parses differently
     OverloadedRecordUpdate, -- qualified fields are not supported
-    OverloadedLabels -- a#b is parsed differently
+    OverloadedLabels, -- a#b is parsed differently
+    ExtendedLiterals, -- 1#Word32 is parsed differently
+    MultilineStrings -- """""" is parsed differently
   ]
 
 -- | Run a 'GHC.P' computation.
@@ -289,7 +300,7 @@ parsePragmasIntoDynFlags ::
   FilePath ->
   -- | Input for parser
   StringBuffer ->
-  IO (Either String ([GHC.Warn], DynFlags))
+  IO (Either String (GHC.DriverMessages, DynFlags))
 parsePragmasIntoDynFlags flags extraOpts filepath input =
   catchGhcErrors $ do
     let (_warnings, fileOpts) =
