@@ -1,3 +1,4 @@
+{-# LANGUAGE DataKinds #-}
 {-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE ViewPatterns #-}
@@ -13,11 +14,17 @@ module Ormolu.Printer.Meat.Common
     p_hsDoc,
     p_hsDocName,
     p_sourceText,
+    p_namespaceSpec,
+    p_arrow,
   )
 where
 
 import Control.Monad
+import Data.Choice (Choice)
+import Data.Choice qualified as Choice
 import Data.Text qualified as T
+import GHC.Data.FastString
+import GHC.Hs.Binds
 import GHC.Hs.Doc
 import GHC.Hs.Extension (GhcPs)
 import GHC.Hs.ImpExp
@@ -27,6 +34,7 @@ import GHC.Types.Name.Occurrence (OccName (..), occNameString)
 import GHC.Types.Name.Reader
 import GHC.Types.SourceText
 import GHC.Types.SrcLoc
+import Language.Haskell.Syntax (HsArrowOf (..))
 import Language.Haskell.Syntax.Module.Name
 import Ormolu.Config (SourceType (..))
 import Ormolu.Printer.Combinators
@@ -52,6 +60,10 @@ p_hsmodName mname = do
 p_ieWrappedName :: IEWrappedName GhcPs -> R ()
 p_ieWrappedName = \case
   IEName _ x -> p_rdrName x
+  IEDefault _ x -> do
+    txt "default"
+    space
+    p_rdrName x
   IEPattern _ x -> do
     txt "pattern"
     space
@@ -65,16 +77,16 @@ p_ieWrappedName = \case
 p_rdrName :: LocatedN RdrName -> R ()
 p_rdrName l = located l $ \x -> do
   unboxedSums <- isExtensionEnabled UnboxedSums
-  let wrapper = \case
-        EpAnn {anns} -> case anns of
-          NameAnnQuote {nann_quoted} -> tickPrefix . wrapper (ann nann_quoted)
-          NameAnn {nann_adornment = NameParens} ->
-            parens N . handleUnboxedSumsAndHashInteraction
-          NameAnn {nann_adornment = NameBackquotes} -> backticks
-          -- special case for unboxed unit tuples
-          NameAnnOnly {nann_adornment = NameParensHash} -> const $ txt "(# #)"
-          _ -> id
-        EpAnnNotUsed -> id
+  let wrapper EpAnn {anns} = case anns of
+        NameAnnQuote {nann_quoted} -> tickPrefix . wrapper nann_quoted
+        NameAnn {nann_adornment = NameParens {}} ->
+          parens N . handleUnboxedSumsAndHashInteraction
+        NameAnn {nann_adornment = NameBackquotes {}} -> backticks
+        -- whether the `->` identifier is parenthesized
+        NameAnnRArrow {nann_mopen = Just _} -> parens N
+        -- special case for unboxed unit tuples
+        NameAnnOnly {nann_adornment = NameParensHash {}} -> const $ txt "(# #)"
+        _ -> id
 
       -- When UnboxedSums is enabled, `(#` is a single lexeme, so we have to
       -- insert spaces when we have a parenthesized operator starting with `#`.
@@ -85,7 +97,7 @@ p_rdrName l = located l $ \x -> do
             \y -> space *> y <* space
         | otherwise = id
 
-  wrapper (ann . getLoc $ l) $ case x of
+  wrapper (getLoc l) $ case x of
     Unqual occName ->
       atom occName
     Qual mname occName ->
@@ -149,7 +161,7 @@ p_hsDoc ::
   -- | Haddock style
   HaddockStyle ->
   -- | Finish the doc string with a newline
-  Bool ->
+  Choice "endNewline" ->
   -- | The 'LHsDoc' to render
   LHsDoc GhcPs ->
   R ()
@@ -172,7 +184,7 @@ p_hsDoc hstyle needsNewline (L l str) = do
       else newline >> txt "--"
     space
     unless (T.null x) (txt x)
-  when needsNewline newline
+  when (Choice.isTrue needsNewline) newline
   case l of
     UnhelpfulSpan _ ->
       -- It's often the case that the comment itself doesn't have a span
@@ -188,4 +200,20 @@ p_hsDocName name = txt ("-- $" <> T.pack name)
 p_sourceText :: SourceText -> R ()
 p_sourceText = \case
   NoSourceText -> pure ()
-  SourceText s -> txt (T.pack s)
+  SourceText s -> atom @FastString s
+
+p_namespaceSpec :: NamespaceSpecifier -> R ()
+p_namespaceSpec = \case
+  NoNamespaceSpecifier -> pure ()
+  TypeNamespaceSpecifier _ -> txt "type" *> space
+  DataNamespaceSpecifier _ -> txt "data" *> space
+
+p_arrow :: (mult -> R ()) -> HsArrowOf mult GhcPs -> R ()
+p_arrow p_mult = \case
+  HsUnrestrictedArrow _ -> txt "->"
+  HsLinearArrow _ -> txt "%1 ->"
+  HsExplicitMult _ mult -> do
+    txt "%"
+    p_mult mult
+    space
+    txt "->"
